package mue.test;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import robocode.control.BattleSpecification;
import robocode.control.BattlefieldSpecification;
import robocode.control.RobocodeEngine;
import robocode.control.RobotSpecification;

/**
 * Runs a series of robocode battles. The challeners will fight NUMBER_OF_RUNS battles (consisting
 * of NUMBER_OF_ROUNDS rounds each) versus each opponent and the results are written to files.
 * 
 * @author mue
 */
public class RunBattles {
	
	// number of runs, so the total number of rounds is NUMBER_OF_ROUNDS * NUMBER_OF_ROUNDS
	// allows to obtain reliable results while still using the number of rounds per battle that 
	// RoboRumble uses  
	protected static int NUMBER_OF_RUNS = 25;

	// number of rounds each battle has (35 is the number used in RoboRumble)
	protected static int NUMBER_OF_ROUNDS = 35;
	
	// if true no challenger will ever face itself in battle
	protected static boolean NO_MIRROR_BATTLES = true;

	// the robots whose results shall be determined
	protected static String[] CHALLENGERS = {
//		"develop.hyperion.Hyperion 0.8",
		"develop.ascendant.Ascendant 1.2.6",
//		"develop.ascendant.AscendantRRGC 1.0.6",
//		"develop.survivor.Survivor 0.7",
	};
	
	// the opponents for the challenger robots 
	protected static String[] OPPONENTS = {
// bots using adaptive movement (mostly wave surfing) 		
// 		"pez.rumble.CassiusClay 1.9.9.93", // 2071		
//		"abc.tron3.Shadow 3.55.1",         // 2065
//		"jam.RaikoMX 0.32",                // 2059
//		"axeBots.SilverSurfer 2.61",       // 2049
//		"cf.proto.Shiva 2.1",              // 2022
//		"rz.Aleph 0.33",                   // 1995
//	  "pe.SandboxDT 3.02",               // 1978
//		"cx.Lacrimas 1.36",                // 1926

// default test field
		"pez.rumble.CassiusClay 1.9.9.93", // 2071
		"cx.Lacrimas 1.36",                // 1926
		"pez.mini.Tityus 0.9.1",           // 1887
		"jam.micro.RaikoMicro 1.44",       // 1885
		"ad.Quest 0.10",                   // 1838
		"kawigi.sbf.FloodMini 1.4",        // 1801
    "emp.Yngwie 1.11",                 // 1712
    "myl.micro.NekoNinja 1.30",        // 1664
		"jw.Booring 1.11",                 // 1647
		"mz.NanoDeath 2.56",               // 1635
		"wiki.nano.DevilFISH 1.0",         // 1564

// default gun test field
//    "pez.rumble.CassiusClay 1.9.9.93", // 2071		
//		"cx.Lacrimas 1.36",                // 1926
//		"pez.mini.Tityus 0.9.1",           // 1887
//		"jam.micro.RaikoMicro 1.44",       // 1885
//		"ad.Quest 0.10",                   // 1838
//		"kawigi.sbf.FloodMini 1.4",        // 1801
//		"emp.Yngwie 1.11",                 // 1712
//		"tzu.TheArtOfWar 1.2",             // 1666
//		"myl.micro.NekoNinja 1.30",        // 1664
//		"jw.Booring 1.11",                 // 1647
//		"tobe.Relativity 3.9",             // 1573

// targeting challenge
//		"pe.SandboxDT 1.91",
//		"apv.AspidMovement 1.0",
//		"tzu.TheArtOfWar 1.2TC",
//		"dummy.micro.Sparrow 2.5TC",
//		"cx.mini.Cigaret 1.31TC",
//		"abc.Tron 2.01",
//		"kawigi.mini.Fhqwhgads 1.1TC",
//		"wiki.etc.HTTC 1.0",
//		"emp.Yngwie 1.0",
//		"kawigi.nano.FunkyChickenTC 1.0",

// sample bots
//    "sample.Corners",
//    "sample.Crazy",
//    "sample.Fire",
//    "sample.MyFirstRobot",
//    "sample.RamFire",
//    "sample.SittingDuck",
//    "sample.SpinBot",
//    "sample.Target",
//    "sample.Tracker",
//    "sample.TrackFire",
//    "sample.Walls",

// own bots
//		"develop.sniper.Sniper 1.0",		
//		"develop.ram.RamBot 1.0",
//		"develop.dominator.Dominator 1.0",
//		"develop.hyperion.Hyperion 0.8",
//		"develop.ascendant.Ascendant 1.2.6",
//		"develop.survivor.Survivor 0.7",
	};
	
	/**
	 * Running the engine.
	 * 
	 * @param args the command line parameters. If existent the first parameter
	 * specifies a filename where to store the results.  
	 */
	public static void main(String args[]) {
		RobocodeEngine engine = null;
		try {
			System.out.print("starting engine... ");
			RobocodeListener listener;
			listener = new RobocodeListener(true,false);
			engine = new RobocodeEngine(listener);
			System.out.println("started.");
			System.out.println("version: "  + engine.getVersion());
			
			System.out.println("reading repository... ");
			RobotSpecification[] repository = engine.getLocalRepository();
			Map robots = new HashMap(repository.length * 2); 
			for (int i = 0; i < repository.length; i++) {
				robots.put(RobocodeListener.getRobotId(repository[i]),repository[i]);
				System.out.println("  found: " + RobocodeListener.getRobotId(repository[i]));
			}
			System.out.println("done.");
			
			boolean validated = true;
			System.out.println("validating robots... ");
			for (int i = 0; i < CHALLENGERS.length; i++) {
				if (!robots.containsKey(CHALLENGERS[i])) {
					System.out.println("challenger " + CHALLENGERS[i] + " not found.");
					validated = false; 
				}
			}
			int battles = OPPONENTS.length * CHALLENGERS.length * NUMBER_OF_RUNS;
			int challengersFound = 0;
			for (int i = 0; i < OPPONENTS.length; i++) {
				if (NO_MIRROR_BATTLES) {
					for (int j = 0; j < CHALLENGERS.length; j++) {
						if (OPPONENTS[i].equals(CHALLENGERS[j])) {
							challengersFound++;
							battles -= NUMBER_OF_RUNS * challengersFound;
						}
					}
				}
				if (!robots.containsKey(OPPONENTS[i])) {
					System.out.println("opponent " + OPPONENTS[i] + " not found.");
					validated = false;
				}
			}
			if (validated) System.out.println("done."); else {
				System.out.println("validation failed... aborting");
				System.exit(1);
			}
			
			long startTime = System.currentTimeMillis();
			System.out.println("scheduled " + battles	+ " battles with " + NUMBER_OF_ROUNDS	+ " rounds each");
			
			System.out.println("waiting 5 seconds to allow to set CPU affinity of this process");
			System.out.println("(Robocode does not like Hyperthreading)");
			Thread.sleep(5000);
			
			// run battles
			BattleSpecification battle;
			BattlefieldSpecification battlefield = new BattlefieldSpecification();
			RobotSpecification[] battleRobots = new RobotSpecification[2];
			int battlesRemaining = battles;
			// iterate over challengers
			for (int challengerIndex = 0; challengerIndex < CHALLENGERS.length; challengerIndex++) {
				battleRobots[0] = (RobotSpecification)robots.get(CHALLENGERS[challengerIndex]);
				
				for (int j = 0; j < NUMBER_OF_RUNS; j++) {
					
					// iterate over the opponents
					for (int i = 0; i < OPPONENTS.length; i++) {
						// make sure no challenger faces itself (if NO_MIRROR_BATTLES is set)
						boolean invalid = false;
						if (NO_MIRROR_BATTLES) {
							for (int k = 0; k <= challengerIndex; k++) if (OPPONENTS[i].equals(CHALLENGERS[k])) {
								invalid = true;
								break;
							}
						}
						if (invalid) continue;

						// run the battle
						battlesRemaining--;
						battleRobots[1] = (RobotSpecification)robots.get(OPPONENTS[i]);
						battle = new BattleSpecification(NUMBER_OF_ROUNDS,battlefield,battleRobots);
						System.out.println("running " + CHALLENGERS[challengerIndex] + " vs " 
						+ OPPONENTS[i] + " (" + battlesRemaining + " battles remaining)");
						engine.runBattle(battle);
						listener.waitForBattle();
					}

					// write results
					String filename = CHALLENGERS[challengerIndex] + ".log";
					Set key = new HashSet();
					key.add(CHALLENGERS[challengerIndex]);
					boolean firstResult = true;
					for (int i = 0; i < OPPONENTS.length; i++) {
						if (NO_MIRROR_BATTLES && CHALLENGERS[challengerIndex].equals(OPPONENTS[i])) continue;
						key.add(OPPONENTS[i]);
						listener.write(key,filename,!firstResult);
						firstResult = false;
						key.remove(OPPONENTS[i]);
					}
					listener.writeEvaluation(CHALLENGERS[challengerIndex],filename,true);
				}
			}
			
			long duration = System.currentTimeMillis() - startTime;
			long hours = duration / 3600000;
			long minutes = (duration - (hours * 3600000)) / 60000;
			long seconds = (duration - (hours * 3600000) - (minutes * 60000)) / 1000;
			System.out.println("executed " + battles + " battles in " 
			+ hours + "h " + minutes + "m " + seconds + "s");
			
		} catch (Exception e) {
			System.out.println("encountered exception: " + e.getMessage());
			e.printStackTrace();
		} finally {
			if (engine != null) engine.close();
			System.out.println("engine closed.");
		}
	}
}